@@ -1,10 +1,11 @@
-import cloudinary
-from cloudinary import utils
-import urllib
-import urllib2
+# Copyright Cloudinary
 import json
 import base64
+import sys
 import email.utils
+import cloudinary
+from cloudinary.compat import urllib2, urlencode, to_string, to_bytes, PY3
+from cloudinary import utils
 
 class Error(Exception): pass
 class NotFound(Error): pass
@@ -62,7 +63,8 @@ def resources_by_ids(public_ids, **options):
     type = options.pop("type", "upload")
     uri = ["resources", resource_type, type]
     params = [("public_ids[]", public_id) for public_id in public_ids]
-    return call_api("get", uri, params + only(options, "tags", "moderations", "context").items(), **options)
+    optional = list(only(options, "tags", "moderations", "context").items())
+    return call_api("get", uri, params + optional, **options)
 
 def resource(public_id, **options):
     resource_type = options.pop("resource_type", "image")
@@ -86,7 +88,8 @@ def delete_resources(public_ids, **options):
     type = options.pop("type", "upload")
     uri = ["resources", resource_type, type]
     params = [("public_ids[]", public_id) for public_id in public_ids]
-    return call_api("delete", uri, params + only(options, "keep_original", "next_cursor").items(), **options)
+    optional = list(only(options, "keep_original", "next_cursor").items())
+    return call_api("delete", uri, params + optional, **options)
 
 def delete_resources_by_prefix(prefix, **options):
     resource_type = options.pop("resource_type", "image")
@@ -98,13 +101,13 @@ def delete_all_resources(**options):
     resource_type = options.pop("resource_type", "image")
     type = options.pop("type", "upload")
     uri = ["resources", resource_type, type]
+    optional = list(only(options, "keep_original", "next_cursor").items())
     return call_api("delete", uri, dict(only(options, "keep_original", "next_cursor"), all=True), **options)
 
-
 def delete_resources_by_tag(tag, **options):
     resource_type = options.pop("resource_type", "image")
     uri = ["resources", resource_type, "tags", tag]
-    return call_api("delete", uri, only(options, "keep_original"), **options)
+    return call_api("delete", uri, only(options, "keep_original", "next_cursor"), **options)
 
 def delete_derived_resources(derived_resource_ids, **options):
     uri = ["derived_resources"]
@@ -152,11 +155,13 @@ def call_api(method, uri, params, **options):
     api_secret = options.pop("api_secret", cloudinary.config().api_secret)
     if not cloud_name: raise Exception("Must supply api_secret")
 
-    data = urllib.urlencode(params)
+    data = to_bytes(urlencode(params))
     api_url = "/".join([prefix, "v1_1", cloud_name] + uri)
     request = urllib2.Request(api_url, data)
     # Add authentication
-    base64string = base64.encodestring('%s:%s' % (api_key, api_secret)).replace('\n', '')
+    byte_value = to_bytes('%s:%s' % (api_key, api_secret))
+    encoded_value = base64.encodebytes(byte_value) if PY3 else base64.encodestring(byte_value)
+    base64string = to_string(encoded_value).replace('\n', '')
     request.add_header("Authorization", "Basic %s" % base64string)
     request.add_header("User-Agent", cloudinary.USER_AGENT)
     request.get_method = lambda: method.upper()
@@ -164,7 +169,8 @@ def call_api(method, uri, params, **options):
     try:
         response = urllib2.urlopen(request)
         body = response.read()
-    except urllib2.HTTPError, e:
+    except urllib2.HTTPError:
+        e = sys.exc_info()[1]
         exception_class = EXCEPTION_CODES.get(e.code)
         if exception_class:
             response = e
@@ -173,9 +179,11 @@ def call_api(method, uri, params, **options):
             raise GeneralError("Server returned unexpected status code - %d - %s" % (e.code, e.read()))
 
     try:
+        body = to_string(body)
         result = json.loads(body)
-    except Exception, e:
+    except Exception:
         # Error is parsing json
+        e = sys.exc_info()[1]
         raise GeneralError("Error parsing server response (%d) - %s. Got - %s" % (response.code, body, e))
 
     if "error" in result:
